@@ -22,7 +22,7 @@ app.config.from_object(__name__)
 
 # create a peewee database instance -- our models will use this database to
 # persist information
-database = SqliteDatabase(DATABASE)
+database = SqliteDatabase(DATABASE, threadlocals=True)
 
 # model definitions -- the standard "pattern" is to define a base model class
 # that specifies which database to use.  then, any subclasses will automatically
@@ -74,6 +74,12 @@ class Relationship(BaseModel):
     from_user = ForeignKeyField(User, related_name='relationships')
     to_user = ForeignKeyField(User, related_name='related_to')
 
+    class Meta:
+        indexes = (
+            # Specify a unique multi-column index on from/to-user.
+            (('from_user', 'to_user'), True),
+        )
+
 
 # a dead simple one-to-many relationship: one user has 0..n messages, exposed by
 # the foreign key.  because we didn't specify, a users messages will be accessible
@@ -90,9 +96,7 @@ class Message(BaseModel):
 # simple utility function to create tables
 def create_tables():
     database.connect()
-    User.create_table()
-    Relationship.create_table()
-    Message.create_table()
+    database.create_tables([User, Relationship, Message])
 
 # flask provides a "session" object, which allows us to store information across
 # requests (stored by default in a secure cookie).  this function allows us to
@@ -133,9 +137,9 @@ def object_list(template_name, qr, var_name='object_list', **kwargs):
 # shortcut "get" method on model, which retrieves a single object or raises a
 # DoesNotExist exception if no matching object exists
 # http://charlesleifer.com/docs/peewee/peewee/models.html#Model.get)
-def get_object_or_404(model, **kwargs):
+def get_object_or_404(model, *expressions):
     try:
-        return model.get(**kwargs)
+        return model.get(*expressions)
     except model.DoesNotExist:
         abort(404)
 
@@ -175,9 +179,7 @@ def private_timeline():
     # messages where the person who created the message is someone the current
     # user is following.  these messages are then ordered newest-first.
     user = get_current_user()
-    messages = Message.select().where(
-        Message.user << user.following()
-    )
+    messages = Message.select().where(Message.user << user.following())
     return object_list('private_messages.html', messages, 'message_list')
 
 @app.route('/public/')
@@ -191,21 +193,19 @@ def join():
     if request.method == 'POST' and request.form['username']:
         try:
             with database.transaction():
-                # if not, create the user and store the form data on the new model
+                # Attempt to create the user. If the username is taken, due to the
+                # unique constraint, the database will raise an IntegrityError.
                 user = User.create(
                     username=request.form['username'],
                     password=md5(request.form['password']).hexdigest(),
                     email=request.form['email'],
-                    join_date=datetime.datetime.now()
-                )
+                    join_date=datetime.datetime.now())
 
             # mark the user as being 'authenticated' by setting the session vars
             auth_user(user)
             return redirect(url_for('homepage'))
 
         except IntegrityError:
-            # use the .get() method to quickly see if a user with that name exists
-            user = User.get(username=request.form['username'])
             flash('That username is already taken')
 
     return render_template('join.html')
@@ -216,8 +216,7 @@ def login():
         try:
             user = User.get(
                 username=request.form['username'],
-                password=md5(request.form['password']).hexdigest()
-            )
+                password=md5(request.form['password']).hexdigest())
         except User.DoesNotExist:
             flash('The password entered is incorrect')
         else:
@@ -253,7 +252,7 @@ def user_list():
 def user_detail(username):
     # using the "get_object_or_404" shortcut here to get a user with a valid
     # username or short-circuit and display a 404 if no user exists in the db
-    user = get_object_or_404(User, username=username)
+    user = get_object_or_404(User, User.username == username)
 
     # get all the users messages ordered newest-first -- note how we're accessing
     # the messages -- user.message_set.  could also have written it as:
@@ -264,18 +263,22 @@ def user_detail(username):
 @app.route('/users/<username>/follow/', methods=['POST'])
 @login_required
 def user_follow(username):
-    user = get_object_or_404(User, username=username)
-    Relationship.get_or_create(
-        from_user=get_current_user(),
-        to_user=user,
-    )
-    flash('You are now following %s' % user.username)
+    user = get_object_or_404(User, User.username == username)
+    try:
+        with database.transaction():
+            Relationship.create(
+                from_user=get_current_user(),
+                to_user=user)
+    except IntegrityError:
+        pass
+
+    flash('You are following %s' % user.username)
     return redirect(url_for('user_detail', username=user.username))
 
 @app.route('/users/<username>/unfollow/', methods=['POST'])
 @login_required
 def user_unfollow(username):
-    user = get_object_or_404(User, username=username)
+    user = get_object_or_404(User, User.username == username)
     Relationship.delete().where(
         (Relationship.from_user == get_current_user()) &
         (Relationship.to_user == user)
@@ -291,8 +294,7 @@ def create():
         message = Message.create(
             user=user,
             content=request.form['content'],
-            pub_date=datetime.datetime.now()
-        )
+            pub_date=datetime.datetime.now())
         flash('Your message has been created')
         return redirect(url_for('user_detail', username=user.username))
 
