@@ -10,6 +10,7 @@ from peewee import Expression
 from peewee import logger
 from peewee import Node
 from peewee import Param
+from peewee import returns_clone
 from peewee import QueryCompiler
 from peewee import SelectQuery
 from peewee import UUIDField  # For backwards-compatibility.
@@ -34,11 +35,26 @@ class _LookupNode(Node):
     def clone_base(self):
         return type(self)(self.node, list(self.parts))
 
-class JsonLookup(_LookupNode):
+class _JsonLookupBase(_LookupNode):
+    def __init__(self, node, parts, as_json=False):
+        super(_JsonLookupBase, self).__init__(node, parts)
+        self._as_json = as_json
+
+    def clone_base(self):
+        return type(self)(self.node, list(self.parts), self._as_json)
+
+    @returns_clone
+    def as_json(self, as_json=True):
+        self._as_json = as_json
+
+class JsonLookup(_JsonLookupBase):
     _node_type = 'json_lookup'
 
     def __getitem__(self, value):
-        return JsonLookup(self.node, self.parts + [value])
+        return JsonLookup(self.node, self.parts + [value], self._as_json)
+
+class JsonPath(_JsonLookupBase):
+    _node_type = 'json_path'
 
 class ObjectSlice(_LookupNode):
     _node_type = 'object_slice'
@@ -155,17 +171,22 @@ class HStoreField(IndexedField):
 class JSONField(Field):
     db_field = 'json'
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, dumps=None, *args, **kwargs):
         if Json is None:
             raise Exception('Your version of psycopg2 does not support JSON.')
+
+        self.dumps = dumps
         super(JSONField, self).__init__(*args, **kwargs)
 
     def db_value(self, value):
-        return Json(value)
+        return Json(value, dumps=self.dumps)
 
     def __getitem__(self, value):
         return JsonLookup(self, [value])
 
+    def path(self, *keys):
+        return JsonPath(self, keys)
+
 
 OP_HKEY = 'key'
 OP_HUPDATE = 'H@>'
@@ -206,16 +227,31 @@ class PostgresqlExtCompiler(QueryCompiler):
                 part, alias_map, conv)
             lookups.append(part_sql)
             params.extend(part_params)
-        # The last lookup should be converted to text.
-        head, tail = lookups[:-1], lookups[-1]
-        sql = '->>'.join(('->'.join(head), tail))
+
+        if node._as_json:
+            sql = '->'.join(lookups)
+        else:
+            # The last lookup should be converted to text.
+            head, tail = lookups[:-1], lookups[-1]
+            sql = '->>'.join(('->'.join(head), tail))
+
         return sql, params
 
+    def _parse_json_path(self, node, alias_map, conv):
+        sql, params = self.parse_node(node.node, alias_map, conv)
+        if node._as_json:
+            operand = '#>'
+        else:
+            operand = '#>>'
+        params.append('{%s}' % ','.join(map(str, node.parts)))
+        return operand.join((sql, self.interpolation)), params
+
     def get_parse_map(self):
         parse_map = super(PostgresqlExtCompiler, self).get_parse_map()
         parse_map.update(
             object_slice=self._parse_object_slice,
-            json_lookup=self._parse_json_lookup)
+            json_lookup=self._parse_json_lookup,
+            json_path=self._parse_json_path)
         return parse_map
 
 
